@@ -24,7 +24,9 @@ inline CharacterVector gdal_create(CharacterVector filename, CharacterVector dri
   oTargetSRS = new OGRSpatialReference;
   OGRErr target_chk =  oTargetSRS->SetFromUserInput(projection[0]);
   if (target_chk != OGRERR_NONE) {
-    delete oTargetSRS;
+    if (oTargetSRS != nullptr) {
+      delete oTargetSRS;
+    }
     Rcpp::stop("cannot initialize target projection");
   }
   
@@ -64,12 +66,9 @@ inline CharacterVector gdal_create(CharacterVector filename, CharacterVector dri
   poDstDS->SetGeoTransform( adfGeoTransform );
   
   
-
-  poDstDS->SetSpatialRef(oTargetSRS);
-  
-  GDALClose(poDstDS);
-  
-  delete oTargetSRS;
+  if (oTargetSRS != nullptr) {
+    delete oTargetSRS;
+  }
   return Rcpp::CharacterVector::create(filename[0]);
 }
 inline CharacterVector gdal_create_copy(CharacterVector dsource, CharacterVector dtarget, CharacterVector driver) {
@@ -106,7 +105,7 @@ inline CharacterVector gdal_create_copy(CharacterVector dsource, CharacterVector
 }
 inline List gdal_read_block(CharacterVector dsn, IntegerVector offset,
                             IntegerVector dimension, IntegerVector band,
-                            CharacterVector band_output_type) {
+                            CharacterVector band_output_type, LogicalVector unscale) {
   IntegerVector window(6);
   window[0] = offset[0];
   window[1] = offset[1];
@@ -114,7 +113,7 @@ inline List gdal_read_block(CharacterVector dsn, IntegerVector offset,
   window[3] = dimension[1];
   window[4] = dimension[0];
   window[5] = dimension[1];
-  return   gdalraster::gdal_raster_io(dsn, window, band, "nearestneighbour", band_output_type);
+  return   gdalraster::gdal_raster_io(dsn, window, band, "nearestneighbour", band_output_type, unscale);
 }
 
 inline LogicalVector gdal_write_block(CharacterVector dsn, NumericVector data,
