@@ -37,6 +37,12 @@ CharacterVector proj_to_wkt_gdal_cpp(CharacterVector proj4string) {
   return gdallibrary::gdal_proj_to_wkt(proj4string);
 }
 
+// [[Rcpp::export]]
+LogicalVector crs_is_lonlat_cpp(CharacterVector input_string) {
+  return gdallibrary::gdal_crs_is_lonlat(input_string);
+}
+
+
 // [[Rcpp::export]]
 LogicalVector register_gdal_cpp() {
   gdallibrary::gdal_register_all();
